#!/usr/bin/env python3
import cgi
import json
import hashlib
from time import sleep
from datetime import datetime
from html import escape
form = cgi.FieldStorage()
demandes = json.load(open("data/demandes-citations.json"))
collecteurs = json.load(open("lib/membres-hgpa.json"))
print('Content-type: text/html\r\n\r')

def sha256(s) : # Retourne le sha256sum du string fournit en base 16 (string).
    hash_var = hashlib.new("sha256")
    hash_var.update(s.encode())
    return hash_var.hexdigest()

def erreur(message) :
    print('ERREUR : ' + message)
    print("Redirection vers la page précédente dans 20/3 secondes...")
    print("Votre navigateur ne supporte pas les redirections ? <a href='../'>Cliquez ici</a>")
    print('<html><head><meta http-equiv="refresh" content="0; url=\'../\'></head></html>')
    exit()

# Retourne un string pouvant sécuritairement être mis dans le html. Retourne une erreur si ce n'est pas possible.
def interpretation_input(variable) :
    if type(variable) is not str or len(variable) == 0 :
        raise EOFError
    return escape(variable)

try:
    type_requete = interpretation_input(form.getvalue("type-requete"))
except:
    erreur("Veuillez spécifier le type de requête souhaité (type-requete)... Gargamel ne sait pas ce que vous voulez !")

if type_requete == "soumettre-citation" :
    demande = {} # La citation demandée.
    # D'abord, on prend les variables requises
    try:
        demande["citation"] = interpretation_input(form.getvalue("citation"))
        if len(demande["citation"]) > 5000 :
            erreur("Votre  est plus grande que 5000 charactères.")
    except:
        erreur("Vous avez omis la citation.")
    try:
        demande["professeur"] = interpretation_input(form.getvalue("professeur"))
        if len(demande["professeur"]) > 500 :
            erreur("Votre professeur est plus grand que 500 charactères.")
    except:
        erreur("Vous avez omis le professeur.")
    try:
        demande["cours"] = interpretation_input(form.getvalue("cours"))
        if len(demande["cours"]) > 500 :
            erreur("Votre  est plus grand que 500 charactères.")
    except:
        erreur("Vous avez omis le cours.")
    try:
        demande["categorie"] = interpretation_input(form.getvalue("categorie"))
        if len(demande["categorie"]) > 1000 :
            erreur("Votre catégorie est plus grande que 1000 charactères.")
    except:
        erreur("Vous avez omis la catégorie.")
    try:
        demande["date"] = interpretation_input(form.getvalue("date"))
        if len(demande["date"]) > 500 :
            erreur("Votre date / session est plus grande que 500 charactères.")
    except:
        erreur("Vous avez omis le moment de votre récolte.")
    try:
        demande["collecteur"] = interpretation_input(form.getvalue("collecteur"))
        if len(demande["collecteur"]) > 500 :
            erreur("Votre nom est plus grand que 500 charactères.")
    except:
        erreur("Vous avez omis le membre que vous êtes.")

    if demande["collecteur"] in collecteurs : # Authentification du membre.
        try :
            collecteur = collecteurs[demande["collecteur"]]
            mot_de_passe = interpretation_input(form.getvalue("mot-de-passe"))
            if len(mot_de_passe) > 256 or sha256(mot_de_passe) != collecteur["mot-de-passe-hash"] :
                erreur("Le mot de passe fournit pour l'utilisateur est invalide. Tenteriez-vous de vous faire passer pour '" + demande["collecteur"] + "' ???")
        except :
            erreur("Pour être considéré comme '" + demande["collecteur"] + "', vous devez fournir son mot de passe (paramètre mot-de-passe).")
    else :
        erreur("Gargamel ne fait pas confiance au collecteur '" + demande["collecteur"] + "'. Veuillez communiquer avec Gargamel pour obtenir votre place dans le club.")

    demandes.append(demande)
    print('a ', demandes, demande, ' b')
    json.dump(demandes, open('data/demandes-citations.json', 'w'), indent='  ', separators=(', ', ': ')) # On sauvegarde le tout.
    print("<html><body><p>Citation envoyée avec succès ! Vous n'avez qu'à attendre l'approbation d'un membre de l'escouade gargamellifique.</p>" +\
          "<a href='/citations/hgpa'>Retourner dans le Hub gargamellifique</a></body></html>")
elif type(type_requete) is str :
    erreur("Mais que voulez vous dire par '" + type_requete + "' ??? Fournissez un type de requête valide, et cessez de hacker mon serveur !")
else :
    erreur("Vous semblez avoir omis le type de requête (type-requete). Que suis-je supposé faire avec les données fournies ???")
