#!/usr/bin/env python3
import cgi
import json
import hashlib
from time import sleep
from datetime import datetime
from html import escape
form = cgi.FieldStorage()
demandes = json.load(open('data/base-de-donnees.json'))
print('Content-type: text/html\r\n\r')

data = json.load(open("lib/elephants.json"))
titres_valides = data["titres_valides"]
couleurs_valides = data["couleurs_valides"]
etats_valides = data["etats_valides"]
utilisateurs = json.load(open("data/utilisateurs.json"))
#titres_valides = {"Régulier", "Régulière", "Vagabond", "Vagabonde", "Minion du grand Gargamel", "Membre du CGPA", "Maître", "Maîtresse", "Agent en uniforme", 
#                  "Agent en uniforme", "Cuboïde", "Kamel Belbahri", "Smith", "[wtf]"}
#couleurs_valides = {"gris", "gris pâle", "gris très pâle", "gris foncé", "gris très foncé", "rouge", "bleu", "vert", "jaune", "orange", "mauve", "rose", "arc-en-ciel", "magenta", "doré", "pamplemousse"}
#etats_valides = {"marié", "en amour", "triomphant", "dôté de surpuissance", "jaloux", "dépressif", "apathique", "gargamellifique", "constitutionnel", "dans la cuisine"}

def sha256(s) : # Retourne le sha256sum du string fournit en base 16 (string).
    hash_var = hashlib.new("sha256")
    hash_var.update(s.encode())
    return hash_var.hexdigest()
    

def erreur(message) :
    print('ERREUR : ' + message)
    print("Redirection vers la page précédente dans 20/3 secondes...")
    print("Votre navigateur ne supporte pas les redirections ? <a href='../'>Cliquez ici</a>")
    print('<html><head><meta http-equiv="refresh" content="0; url=\'../\'></head></html>')
    exit()


# Retourne un string pouvant sécuritairement être mis dans le html. Retourne une erreur si ce n'est pas possible.
def interpretation_input(variable) :
    if type(variable) is not str or len(variable) == 0 :
        raise EOFError
    return escape(variable)



try:
    type_requete = interpretation_input(form.getvalue("type-requete"))
except:
    erreur("Veuillez spécifier le type de requête souhaité (type-requete)... Gargamel ne sait pas ce que vous voulez !")

if type_requete == "consulter-demande" :
    try:
        demande_no = int(form.getvalue("demande-no"))
        demande = demandes[demande_no]
    except:
        erreur("Vous avez omis le numéro de demande à consulter (demande-no), ou bien il est invalide...")
    print(open('demande-page-haut.html').read())
    # On imprime d'abord la demande.
    demande["nb_reponses"] = str(len(demande["reponses"])) # Pour l'utiliser.
    demande["lien_reponse_demande"] = "#repondre" # Lien relatif pour répondre à la demande.
    demande_txt = open('demande-individuelle.html').read() # Le string qui sera imprimé pour la demande. Pour l'instant, chaque variable 'VARIABLE' est de la forme $VARIABLE.
    for nom_variable in demande :
        if type(demande[nom_variable]) is str :
            demande_txt = demande_txt.replace('$' + nom_variable, demande[nom_variable])
    print(demande_txt)
    # Ensuite, on imprime chaque réponse.
    reponse_layout = open('reponse-individuelle.html').read() # html général d'une réponse; chaque variable 'VARIABLE' est de la forme $VARIABLE.
    for reponse in demande['reponses']: # reponse est un dictionnaire représentant chaque aspect de la réponse.
        reponse_txt = reponse_layout
        for nom_variable in reponse :
            if type(reponse[nom_variable]) is str :
                reponse_txt = reponse_txt.replace('$' + nom_variable, reponse[nom_variable])
        print(reponse_txt)
    try :
        anonyme = "false" != form.getvalue("anonyme")
    except :
        anonyme = True
    if anonyme :
        print(open('demande-page-bas.html').read().replace('$demande_no', str(demande_no)))
    else :
        print(open('utilisateur-demande-page-bas.html').read().replace('$demande_no', str(demande_no)))
    print(open('layout-bas.html').read())

elif type_requete == "nouvelle-demande" :
    demande = {}
    demande_no = str(len(demandes)) # String unique à la demande (position dans la base de données).
    demande["demande_no"] = demande_no
    demande["date"] = datetime.now().strftime("%Y-%m-%d %H:%M")
    # D'abord, on prend les variables requises
    try:
        demande["sujet"] = interpretation_input(form.getvalue("sujet"))
    except:
        erreur("Vous avez omis le sujet.")
    try:
        demande["titre"] = interpretation_input(form.getvalue("titre"))
    except:
        erreur("Vous avez omis votre titre.")
    try:
        demande["simple_couleur"] = interpretation_input(form.getvalue("couleur")) # Classe css de la couleur.
    except:
        erreur("Vous avez omis votre couleur.")
    try:
        demande["etat"] = interpretation_input(form.getvalue("etat"))
    except:
        erreur("Vous avez omis votre état actuel")
    try:
        demande["texte"] = interpretation_input(form.getvalue("texte"))
    except:
        erreur("Vous avez omis votre texte.")
    # Ensuite, on s'assure qu'elles respectent les règles.
    if len(demande["sujet"]) > 1000 :
        erreur("Sérieusement ? Plus de 1000 charactères pour le sujet !? Tentez d'être plus concis svp...")
    if len(demande["texte"]) > 200000 :
        erreur("Je suis désolé, mais je n'ai pas accès à un espace de stockage infini... Soit vous écrivez votre demande d'amour en moins de 200001 charactères, ou bien vous vivrez seul !")
    try:
        demande["titre"] = titres_valides[demande["titre"]]
        demande["fond_couleur"] = "blue"
    except:
        if demande["titre"] in utilisateurs :
            try :
                utilisateur = utilisateurs[demande["titre"]]
                mot_de_passe = interpretation_input(form.getvalue("mot-de-passe"))
                if len(mot_de_passe) > 256 or sha256(mot_de_passe) != utilisateur["mot_de_passe_hash"] :
                    erreur("Le mot de passe fournit pour l'utilisateur est invalide. Tenteriez-vous de vous faire passer pour '" + demande["titre"] + "' ???")
                demande["fond_couleur"] = utilisateur["fond_couleur"]
                demande["titre"] = utilisateur["titre"]
            except :
                erreur("Pour être considéré comme '" + utilisateurs["titre"] + "', vous devez fournir son mot de passe (paramètre mot-de-passe).")
        else :
            erreur("Gargamel ne croit pas en l'existence d'un '" + demande["titre"] + "'. Veuillez adopter un titre valide.")
    try:
        demande["couleur"] = couleurs_valides[demande["simple_couleur"]]
    except:
        erreur("Votre couleur de peau est formelement refusée par le système.")
    try:
        demande["etat"] = etats_valides[demande["etat"]]
    except:
        erreur("Votre état actuel vous est refusé. Veuillez changer d'attitude pour participer sur ce forum.")
    demande["reponses"] = []
    demandes.append(demande)
    json.dump(demandes, open('data/base-de-donnees.json', 'w'), indent='  ', separators=(', ', ': ')) # On sauvegarde le tout.
    # On finit en rafraîchissant la page, pour que l'utilisateur puisse voir sa demande :
    print('<html><head><meta http-equiv="refresh" content="0; url=\'index.cgi#demande-' + demande_no + '\'"></head></html>')
elif type_requete == "reponse-demande" :
    try:
        demande_no = int(form.getvalue("demande-no"))
    except:
        erreur("Vous avez omis le numéro de demande à laquelle vous souhaitez répondre (demande-no), ou bien il est invalide...")
    reponse = {}
    reponse_no = str(len(demandes)) # String unique à la réponse, relativement à la demande associée (position parmis les réponses de la demande).
    reponse["reponse_no"] = reponse_no
    reponse["date"] = datetime.now().strftime("%Y-%m-%d %H:%M")
    # D'abord, on prend les variables requises
    try:
        reponse["titre"] = interpretation_input(form.getvalue("titre"))
    except:
        erreur("Vous avez omis votre titre.")
    try:
        reponse["simple_couleur"] = interpretation_input(form.getvalue("couleur")) # Classe css de la couleur.
    except:
        erreur("Vous avez omis votre couleur.")
    try:
        reponse["etat"] = interpretation_input(form.getvalue("etat"))
    except:
        erreur("Vous avez omis votre état actuel")
    try:
        reponse["texte"] = interpretation_input(form.getvalue("texte"))
    except:
        erreur("Vous avez omis votre texte.")
    # Ensuite, on s'assure qu'elles respectent les règles.
    if len(reponse["texte"]) > 200000 :
        erreur("Je suis désolé, mais je n'ai pas accès à un espace de stockage infini... Soit vous écrivez votre demande d'amour en moins de 200001 charactères, ou bien vous vivrez seul !")

    try:
        reponse["titre"] = titres_valides[reponse["titre"]]
        reponse["fond_couleur"] = "red"
    except:
        if reponse["titre"] in utilisateurs :
            try :
                utilisateur = utilisateurs[reponse["titre"]]
                mot_de_passe = interpretation_input(form.getvalue("mot-de-passe"))
                if len(mot_de_passe) > 256 or sha256(mot_de_passe) != utilisateur["mot_de_passe_hash"] :
                    erreur("Le mot de passe fournit pour l'utilisateur est invalide. Tenteriez-vous de vous faire passer pour '" + reponse["titre"] + "' ???")
                reponse["fond_couleur"] = utilisateur["fond_couleur"]
                reponse["titre"] = utilisateur["titre"]
            except :
                erreur("Pour être considéré comme '" + utilisateurs["titre"] + "', vous devez fournir son mot de passe (paramètre mot-de-passe).")
        else :
            erreur("Gargamel ne croit pas en l'existence d'un '" + reponse["titre"] + "'. Veuillez adopter un titre valide.")
    try:
        reponse["couleur"] = couleurs_valides[reponse["simple_couleur"]]
    except:
        erreur("Votre couleur de peau est formelement refusée par le système.")
    try:
        reponse["etat"] = etats_valides[reponse["etat"]]
    except:
        erreur("Votre état actuel vous est refusé. Veuillez changer d'attitude pour participer sur ce forum.")
    demandes[demande_no]["reponses"].append(reponse)
    json.dump(demandes, open('data/base-de-donnees.json', 'w'), indent='  ', separators=(', ', ': ')) # On sauvegarde le tout.
    # On finit en rafraîchissant la page, pour que l'utilisateur puisse voir sa réponse :
    print('<html><head><meta http-equiv="refresh" content="0; url=\'demande.cgi?type-requete=consulter-demande&demande-no=' +\
            str(demande_no) + '#reponse-no=' + reponse_no + '\'"></head></html>')
elif type(type_requete) is str :
    erreur("Mais que voulez vous dire par '" + type_requete + "' ??? Fournissez un type de requête valide, et cessez de hacker mon serveur !")
else :
    erreur("Vous semblez avoir omis le type de requête (type-requete). Que suis-je supposé faire avec les données fournies ???")
